using Attribute;
using Common;
using Dto;
using Vo;
using Enums;
using Filters;
using Infrastructure;
using Infrastructure.Model;
using Mapster;
using Microsoft.AspNetCore.Mvc;
using Middleware;
using Model;
using Services;
using Model.Base;

namespace Controllers
{
    /// <summary>
    /// SysDictItem)Controller
    /// </summary>
    // [Route("SysDictItem")]
    // [ApiExplorerSettings(GroupName = "SysDictItem")]
    public class SysDictItemController : BaseController
    {
        /// <summary>
        /// api分组接口
        /// </summary>
        private readonly ISysDictItemService _SysDictItemService;


        public SysDictItemController(ISysDictItemService SysDictItemService)
        {
            _SysDictItemService = SysDictItemService;

        }

        /// <summary>
        /// 字典项列表
        /// </summary>
        /// <param name="parm">请求参数</param>
        /// <returns>字典项列表</returns>
        [HttpGet]
        [Route("/v1/api/SysDictItem/list")]
        public IActionResult List([FromQuery] SysDictItemListDto parm)
        {
            var response = _SysDictItemService.List(parm);
            return SUCCESS(response);
        }




        /// <summary>
        /// 字典项详情
        /// </summary>
        /// <param name="parm">请求参数</param>
        /// <returns>字典项详情</returns>
        [HttpGet]
        [Route("/v1/api/SysDictItem/query")]
        public IActionResult Query([FromQuery] SysDictItemQueryDto parm)
        {
            var response = _SysDictItemService.Queryable()
                .First(x => x.id == parm.id)
                .Adapt<SysDictItemQueryVo>() ?? new SysDictItemQueryVo();
            return SUCCESS(response);
        }




        /// <summary>
        /// 字典项添加
        /// </summary>
        /// <param name="parm">请求参数</param>
        /// <returns>字典项添加</returns>
        [HttpPost]
        [Route("/v1/api/SysDictItem/add")]
        public IActionResult Add([FromBody] SysDictItemAddDto parm)
        {
            var modal = parm.Adapt<SysDictItem>().ToCreate(HttpContext);
            var response = _SysDictItemService.Add(modal);
            return SUCCESS(response);
        }




        /// <summary>
        /// 字典项修改
        /// </summary>
        /// <param name="parm">请求参数</param>
        /// <returns>字典项修改</returns>
        [HttpPut]
        [Route("/v1/api/SysDictItem/update")]
        public IActionResult Update([FromBody] SysDictItemUpdateDto parm)
        {
            var modal = parm.Adapt<SysDictItem>().ToUpdate(HttpContext);
            var response = _SysDictItemService.Update(modal, true);
            return SUCCESS(response);
        }




        /// <summary>
        /// 字典项删除
        /// </summary>
        /// <param name="Id">编号</param>
        /// <returns>字典项删除</returns>
        [HttpDelete]
        [Route("/v1/api/SysDictItem/delete/{id}")]
        public IActionResult Delete(long id)
        {
            var response = _SysDictItemService.Delete(id);
            return SUCCESS(response);
        }






    }
}