using Attribute;
using Model;
using Model.Base;
using Repository;
using Service;
using Microsoft.AspNetCore.Mvc;
using Vo.Admin;


namespace Services
{
    /// <summary>
    /// 数据库连接配置Service业务层处理
    /// </summary>
    [AppService(ServiceType = typeof(IPriDatabaseSetService), ServiceLifetime = LifeTime.Transient)]
    public class PriDatabaseSetService : BaseService<PriDatabaseSet>, IPriDatabaseSetService
    {
        /// <summary>
        /// 数据库连接配置-列表
        /// </summary>
        /// <param name="param">参数请求体</param>
        /// <param name="page">分页参数</param>
        /// <returns>列表</returns>
        public PagedInfo<GetPriDatabaseSetListVo> getPriDatabaseSetList([FromQuery] PagerInfo page, [FromQuery] PriDatabaseSet param)
        {
            //拼装查询条件
            var predicate = Expressionable.Create<PriDatabaseSet>();
            predicate = predicate.AndIF(!string.IsNullOrEmpty(param.ipAddress), m => m.ipAddress.Contains(param.ipAddress));
            predicate = predicate.AndIF(!string.IsNullOrEmpty(param.userId), m => m.userId.Contains(param.userId));
            predicate = predicate.AndIF(param.projectId > 0, m => m.projectId == param.projectId);
            var response = Queryable()
                .Where(predicate.ToExpression())
                .OrderByDescending(m => m.id)
                .ToPage<PriDatabaseSet, GetPriDatabaseSetListVo>(page);
            return response;
        }


        /// <summary>
        /// 数据库连接配置-下拉框数据
        /// </summary>
        /// <param name="page">分页参数</param>
        /// <returns>下拉框数据</returns>
        public PagedInfo<GetPriDatabaseSetDicVo> getPriDatabaseSetDic([FromQuery] PagerInfo page)
        {
            //拼装查询条件
            var predicate = Expressionable.Create<PriDatabaseSet>();
            var response = Queryable()
                .Where(predicate.ToExpression())
                .OrderByDescending(m => m.id)
                .ToPage<PriDatabaseSet, GetPriDatabaseSetDicVo>(page);
            return response;
        }



    }
}
